MoneyDetailFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0")

MoneyDetailFu.version = "2.0." .. string.sub("$Revision: 75231 $", 12, -3)
MoneyDetailFu.date = string.sub("$Date: 2008-05-27 06:43:34 +0200 (Tue, 27 May 2008) $", 8, 17)
MoneyDetailFu.hasIcon = true
MoneyDetailFu.clickableTooltip = true
MoneyDetailFu.tooltipHiddenWhenEmpty = true


MoneyDetailFu:RegisterDB("MoneyDetailFuDB")
MoneyDetailFu:RegisterDefaults('profile', {
	allocateUnkownMoney = 1,
	debugLevel = 0,
	repairCostRange = 10,
	playerIndex =1,
	dayIndex =1,	
	monthIndex ="January",	
	yearIndex =2007,
	showAllRealmMoney =1,
})

local tablet = AceLibrary("Tablet-2.0")
local abacus = LibStub("LibAbacus-3.0")
local deformat = AceLibrary("Deformat-2.0")

local L = AceLibrary("AceLocale-2.2"):new("FuBar_MoneyDetailFu")

local mType_Loot = 1;
local mType_Repair = 2;
local mType_Buy = 3;
local mType_Sell = 4;
local mType_Quest = 5;
local mType_MailSent = 6;
local mType_MailCost = 7;
local mType_MailCODCost = 13;
local mType_MailCODIncome = 15;
local mType_MailMoneyReceived = 14;
local mType_Fly = 8;
local mType_AuctionCost = 9;
local mType_AuctionBid = 10;
local mType_AuctionOutbid = 11;
local mType_AuctionSold = 12;
local mType_Talents = 16;

local auction_bidPlaced =1;
local auction_auctionCreated =2;

local const_allPlayers = 20; -- constant passed as index for all players

function MoneyDetailFu:OnEnable()
	self.hasIcon = true
	local frame = self.frame
	local icon = frame:CreateTexture("MoneyFuFrameIcon", "ARTWORK")
	icon:SetWidth(16)
	icon:SetHeight(16)
	icon:SetPoint("LEFT", frame, "LEFT")
	self.iconFrame = icon
	self.player = UnitName("player");
	self.realm = GetRealmName();
	
	--self.realm = "Kazzak";
	--self.player = "Sandrielle";
	
	local recreate = nil;
	
	if (self.player) then
		self:Debug("Player "..self.player,5);
	end
	if (self.realm) then
		self:Debug("Realm "..self.realm,5);
	end	
	if (not self.db.profile.data) then
		self.db.profile.data = {};
		recreate = 1;
	end
	if (not self.db.profile.data[self.realm]) then
		self.db.profile.data[self.realm] = {};
		recreate = 1;
	end
	if (not self.db.profile.data[self.realm][self.player]) then
		self.db.profile.data[self.realm][self.player] = {};
		recreate = 1;
	end

	self.lastMoney = GetMoney();
	self.merchant = nil;
	self.mail = nil;
	self.AHOpen = nil;
	self.questMoney = nil;
	self.repairCost = 0;
	self.goldLoot = nil;
	self.attachMoney = nil;
	self.taxi = nil;
	self.talentWipe = nil;	
	self.trainer = nil;
	self.auctionAction = nil;
	self.mailUpdateCode = nil;
	self.mailUpdateMoney = 0;
	
	if (not self.db.profile.characters) then
		self.db.profile.characters = {};
	end
	
	if (not self.db.profile.characters[self.realm]) then
		self.db.profile.characters[self.realm] = {};
	end	
		
	if (not self.db.profile.characters[self.realm][self.player]) then
		self.db.profile.characters[self.realm][self.player] = {};
	end
	self.db.profile.characters[self.realm][self.player].money = GetMoney();
	--self.db.profile.characters[self.realm][self.player].played = RequestTimePlayed();	
	
	if (not self.db.profile.indexPlayers) or (recreate) then
		self:CreateIndexLists();
	else
		self:ResetIndexes();
	end
	
	self:RegisterEvent("PLAYER_MONEY", "MoneyChanged")
	self:RegisterEvent("MERCHANT_SHOW", "OpenMerchant")
	self:RegisterEvent("MERCHANT_CLOSED", "CloseMerchant")	
	self:RegisterEvent("MAIL_SHOW", "OpenMail")	
	self:RegisterEvent("MAIL_CLOSED", "CloseMail")		
	self:RegisterEvent("AUCTION_HOUSE_SHOW", "OpenAH")	
	self:RegisterEvent("AUCTION_HOUSE_CLOSED", "CloseAH")		
	self:RegisterEvent("SEND_MAIL_MONEY_CHANGED", "SentMoneyChange")			
	self:RegisterEvent("QUEST_COMPLETE", "QuestMoney")				
	self:RegisterEvent("LOOT_OPENED", "LootOpen")					
	self:RegisterEvent("CHAT_MSG_MONEY", "MoneyMsg")						
	self:RegisterEvent("TAXIMAP_OPENED", "TaxiOpen")						
	self:RegisterEvent("TAXIMAP_CLOSED", "TaxiClosed")							
	self:RegisterEvent("CHAT_MSG_SYSTEM", "ChatSystem")							
	self:RegisterEvent("MAIL_INBOX_UPDATE", "CheckMailUpdate")								
	self:RegisterEvent("TRAINER_SHOW", "OpenTrainer")								
	self:RegisterEvent("TRAINER_CLOSED", "CloseTrainer")								
	self:RegisterEvent("CONFIRM_TALENT_WIPE", "TalentWipe")								-- both are retraining costs
	self:RegisterEvent("CONFIRM_PET_UNLEARN", "TalentWipe")								-- both are retraining costs	
	
	--self:RegisterEvent("QUEST_FINISHED", "QuestMoney")					
	--self:RegisterBucketEvent({"PLAYER_TRADE_MONEY", "TRADE_MONEY_CHANGED","SEND_MAIL_MONEY_CHANGED","SEND_MAIL_COD_CHANGED"}, 5, "EventProcess")
	
	--self:OnClick();
end

local options = {
	type = 'group',
	args = {
		debuggingLevel = {
			type = 'range',
			name = L["Debug Level"],
			desc = L["The amount of debugging messages shown (-1 for nothing at all, 0 for some alert msg, >0 real debugging) !"],
			get = function()
				return MoneyDetailFu.db.profile.debugLevel;
			end,
			set = function(v)
				MoneyDetailFu.db.profile.debugLevel = v
			end,
			min = -1,
			max = 5,
			step = 1,
			order = 10
		},	
		allocateUnkownMoney = {
			type = 'toggle',
			name = L["Allocate All Mail Money"],
			desc = L["If a matching money amount with mail cant be found, income will be allocated to outbid auctions"],
			get = function()
				return MoneyDetailFu.db.profile.allocateUnkownMoney
			end,
			set = function(value)
				MoneyDetailFu.db.profile.allocateUnkownMoney = value;
			end,
			order = 15,
		},
		showAllRealmMoney = {
			type = 'toggle',
			name = L["Show All Realms Money"],
			desc = L["Show Money of all realms in tooltip, otherwise just present realm."],
			get = function()
				return MoneyDetailFu.db.profile.showAllRealmMoney
			end,
			set = function(value)
				MoneyDetailFu.db.profile.showAllRealmMoney = value;
			end,
			order = 20,
		},		
		createIndexLists = {
			order = 25,
			type = "execute",
			name = L["Recreate Index Lists"],
			desc = L["Creates new lists of players / realms you can browse threw."],
			func = "CreateIndexLists",
		},						
	},
	handler = MoneyDetailFu	
}
MoneyDetailFu:RegisterChatCommand(L["AceConsole-Commands"], options)
MoneyDetailFu.OnMenuRequest = options

function MoneyDetailFu:Debug(msg,level)
	if (level <= self.db.profile.debugLevel) then
		if (level <= 0) then
			self:Print(msg);
		else
			self:Print("Debug :"..msg);
		end
	end
end

function MoneyDetailFu:MoneyChanged()
	local current = GetMoney();
	self.db.profile.characters[self.realm][self.player].money = current;
	spentMoney = self.lastMoney - current;
	earnedMoney = spentMoney * -1;
	self:Debug("Last "..self.lastMoney.." Now "..current.." Spent "..spentMoney,2);	
	if (self.merchant) then
		if (CanMerchantRepair()) and (self.repairCost > 0) and (GetRepairAllCost() < self.repairCost) then --repair happened
			local repairAmount = self.repairCost - GetRepairAllCost(); -- for how much should have been repaired
			if (spentMoney >= repairAmount-self.db.profile.repairCostRange) and (spentMoney <= repairAmount+self.db.profile.repairCostRange) then
				-- repair cost withing defined precision range -> only repair
				self:Debug("Repaired for "..spentMoney,1);				
				self:AddData(mType_Repair, spentMoney);
			elseif (spentMoney > repairAmount) then -- repair and bought
				boughtMoney = spentMoney - repairAmount;
				self:Debug("Repaired for "..repairAmount.. " and bought for : "..boughtMoney,1);
				self:AddData(mType_Repair, repairAmount);
				self:AddData(mType_Buy, boughtMoney);							
			elseif (spentMoney < repairAmount) then -- repair and sold
				soldMoney = repairAmount - spentMoney;
				self:Debug("Repaired for "..repairAmount.. " and sold for : "..soldMoney,1);				
				self:AddData(mType_Repair, repairAmount);
				self:AddData(mType_Sell, soldMoney);							
			end			
			self.repairCost = GetRepairAllCost();			
		elseif (spentMoney > 0) then -- bought only			
			self:Debug("Bought for : "..spentMoney,1);
			self:AddData(mType_Buy, spentMoney);			
		elseif (earnedMoney > 0) then -- sold stuff
			self:Debug("Sold for : "..earnedMoney,1);
			self:AddData(mType_Sell, earnedMoney);			
		else
			self:Debug("Undefined merchant repair / buy / sell money amoung !",0); -- should never be reached
		end
	elseif (self.questMoney) and (self.questMoney > 0) and (earnedMoney == self.questMoney) then		-- quest money
		self:Debug("Reward from Quest : "..earnedMoney,1);
		self:AddData(mType_Quest, earnedMoney);					
	elseif (self.goldLoot) and (self.goldLoot == earnedMoney) then -- loot money
		self:Debug("Money looted : "..earnedMoney,1);
		self:AddData(mType_Loot, earnedMoney);		
	elseif (self.mail) then -- money in mailbox
		if (spentMoney > 0) and (spentMoney <= 600) and (math.fmod(spentMoney,30) == 0) then
			self:Debug("Mail sent for : "..spentMoney,1);
			self:AddData(mType_MailCost, spentMoney);									
			self.attachMoney = nil;
		elseif (self.attachMoney) then
			if (spentMoney == self.attachMoney) then
				self:Debug("Money sent : "..spentMoney,1);			
				self:AddData(mType_MailSent, spentMoney);													
			elseif (spentMoney-30 == self.attachMoney) then				
				self:Debug("Money sent : "..spentMoney-30,1);			
				self:Debug("Mail sent for : 30",1);
				self:AddData(mType_MailCost, 30);														
				self:AddData(mType_MailSent, spentMoney-30);						
			end						
		elseif (self.mailUpdateCode) then
			if (earnedMoney > 0) and (self.mailUpdateMoney == earnedMoney) then -- money received via mail from player or COD payment, or outbid on auctions or sold auction
				self:AddData(self.mailUpdateCode, earnedMoney);																
				self:Debug("Money booked on mailUpdateCode : "..earnedMoney,1);				
			else				
				if (self.db.profile.allocateUnkownMoney) and (earnedMoney>0) then
					self:AddData(mType_AuctionOutbid, earnedMoney);																
					self:Debug("Unknown mail amount with UpdateCode booked on  Auction Outbid : "..earnedMoney,1);								
				else				
					self:Debug("Mail update code set, but no matching money !"..earnedMoney,0);			
				end
			end
			self.mailUpdateCode = nil;
			self.mailUpdateMoney = 0;			
		elseif (spentMoney > 0) then -- cod, works without mailUpdateCode since money is spent b4 mail is removed, only cost in mailbox > 30 must be cod				
			self:AddData(mType_MailCODCost, spentMoney);				
			self:Debug("Money spent for COD : "..spentMoney,1);															
			self.mailUpdateCode = nil;
			self.mailUpdateMoney = 0;						
		else			
			if (self.db.profile.allocateUnkownMoney) and (earnedMoney>0) then
				self:AddData(mType_AuctionOutbid, earnedMoney);																
				self:Debug("Unknown mail amount booked on  Auction Outbid : "..earnedMoney,1);								
			else
				self:Debug("Unknown mail money income !"..earnedMoney,0);			
			end
		end		
	elseif (self.trainer) or (self.talentWipe) then
		self:Debug("Training / Talent Wipe !"..spentMoney,1);		
		self:AddData(mType_Talents, spentMoney);									
		self.talentWipe = nil;		
		self:Update();
	elseif (self.taxi) then
		self:Debug("Taxi stand !",3);
		if (UnitOnTaxi("player")) then
			self:Debug("Flying !",3);
			self:Debug("Taxi cost : "..spentMoney,1);			
			self:AddData(mType_Fly, spentMoney);									
			self.taxi = nil;
		end
	elseif (self.AHOpen) then		
		if (self.auctionAction == auction_bidPlaced) then
			self:Debug("Auction Bid Cost : "..spentMoney,1);			
			self:AddData(mType_AuctionBid, spentMoney);												
		elseif (self.auctionAction == auction_auctionCreated) then
			self:Debug("Auction cost : "..spentMoney,1);			
			self:AddData(mType_AuctionCost, spentMoney);												
		else
			self:Debug("Unknown auction money spent : "..spentMoney,0);						
		end		
		self.auctionAction = nil;
	else
		if (self.goldLoot) and (self.db.profile.allocateUnkownMoney) and (earnedMoney > 0) and (earnedMoney > self.goldLoot) then
			self:Debug("Unknown Money loot assigned : "..earnedMoney,1);
			self:AddData(mType_Loot, earnedMoney);		
		else			
			if (UnitOnTaxi("player")) and (self.db.profile.allocateUnkownMoney) then
				self:Debug("Unknown cost booked on taxi since flying : "..spentMoney,1);			
				self:AddData(mType_Fly, spentMoney);													
			else
				self:Debug("Money cost unknown reason : "..spentMoney,0);			
			end
		end
	end
	self.lastMoney = current;	
	self.questMoney = nil;
	self.attachMoney = nil;
	self.goldLoot = nil;
end

function MoneyDetailFu:OpenMerchant()
	self.merchant = 1;
	if CanMerchantRepair() then
		self.repairCost = GetRepairAllCost()
		self:Debug("Repair possible : "..self.repairCost,3);
	end	
	self:Update();
end

function MoneyDetailFu:CloseMerchant()
	self.merchant = nil;
	self:Update();
end

function MoneyDetailFu:OpenMail()
	self.mail = 1;
	self.mailUpdateCode = nil;
	self:Update();
end

function MoneyDetailFu:CloseMail()
	self.mail = nil;
	self.attachMoney = nil;
	self.mailUpdateCode = nil;
	self:Update();
end

function MoneyDetailFu:OpenAH()
	self.AHOpen = 1;
	self.auctionAction = nil;
	self:Update();
end

function MoneyDetailFu:CloseAH()
	self.AHOpen = nil;
	self.auctionAction = nil;
	self:Update();
end

function MoneyDetailFu:TaxiOpen()
	self.taxi = 1;
	self:Update();
end

function MoneyDetailFu:TaxiClosed()	
	self:ScheduleEvent("TaxiRecentClosed", self.ResetTaxiStand, 2,self);
end

function MoneyDetailFu:ResetTaxiStand()
	self.taxi = nil;
	self:Update();	
end

function MoneyDetailFu:OpenTrainer()
	if not UnitExists('npc') or UnitPlayerControlled('npc') then
		return
	end
	self.trainer = 1;
	self:Update();
end

function MoneyDetailFu:CloseTrainer()
	self.trainer = nil;
	self:Update();	
end

function MoneyDetailFu:TalentWipe()
	self:Debug("Talent Wipe !",1);	
	self:ScheduleEvent("MDF_ResetTalenWipe", self.ResetTalenWipe, 10,self);
	self.talentWipe = 1;
	self:Update();	
end

function MoneyDetailFu:ResetTalenWipe()
	self:Debug("Reset Wipe !",1);
	self.talentWipe = nil;
	self:Update();	
end

function MoneyDetailFu:SentMoneyChange()
	local codAmount = GetSendMailMoney();
	if (codAmount) and (codAmount > 0) then
		self:Debug("Amount : "..codAmount,3);
		self.attachMoney = codAmount;
	end
end

function MoneyDetailFu:QuestMoney()
	self.questMoney = GetRewardMoney();	
	self:Debug("Quest Reward Money "..self.questMoney,3);
end

function MoneyDetailFu:LootOpen()
 	for index = 1, GetNumLootItems(), 1 do
  	if (LootSlotIsCoin(index)) then
    	local _,coinName = GetLootSlotInfo(index);
    	coinName = string.gsub(coinName, "\n", " ", 1, true);
      self:Debug("Money in loot "..coinName,3);
      self:MoneyMsg(coinName);
    end
  end
end

function MoneyDetailFu:CheckMailUpdate()
	if (self.mailData) and (table.getn(self.mailData) > 0) then
		self:MailUpdate();		
	else
		self:SaveMailData();
	end	
end

function MoneyDetailFu:MailUpdate()
	local index = GetInboxNumItems();	
	self:Debug("Number inb items : "..index,5);
	if index > 0 then	
		for mi=1,index do				
			local _, _, sender, subject, money, CODAmount, _, hasItem, _, _, _, _ = GetInboxHeaderInfo(mi);
			if (money > 0) or (CODAmount > 0) then
				if (money > 0) then
					CODAmount = 1234567890; -- if money attached, cod amount does not matter, only using money for later compare ... "or" criteria
				else
					money = 1234567890; -- if cod, money does not matter, only using cod for later compare ... "or" criteria
				end
	      self:Debug("Index : "..mi.." send : "..sender.." money :"..money.." sender : "..sender,5);				
				local num = table.getn(self.mailData);				
				local i = 0;
				while i < num do
					i = i+1;
					if ( (money == self.mailData[i].money) or (CODAmount == self.mailData[i].cod) ) and (subject == self.mailData[i].subject) and (sender == self.mailData[i].sender) then -- mail found
						self:Debug("Found. Removing Index : "..i,5);				
						table.remove(self.mailData,i);
						i =num; -- finish loop, remove only 1
					end
				end
			end			
		end
	end	
	local num = table.getn(self.mailData);
	self:Debug("Remaining non found mail Items : "..num,3);			-- should be only 1 remaining then this one was picked up
	if (num == 1) then		
		self:Debug("Evaluation money source !",3);
		self.mailUpdateMoney = self.mailData[1].money;
		if (self.mailData[1].money > 0) then
			if (self.mailData[1].sender == L["Alliance Auction House"]) or (self.mailData[1].sender == L["Horde Auction House"]) then		
				if (string.find(self.mailData[1].subject, L["Auction successful: (.+)"])) then
					self.mailUpdateCode = mType_AuctionSold;
					self:Debug("Auction sold for !"..self.mailUpdateMoney,1);									
				elseif (string.find(self.mailData[1].subject, L["Outbid on (.+)"])) then
					self.mailUpdateCode = mType_AuctionOutbid;
					self:Debug("Outbid on auction. Money back : !"..self.mailUpdateMoney,1);													
				else
					self:Debug("Unknown mail with money received from AH !",0);
				end			
			elseif (string.find(self.mailData[1].subject, L["COD Payment:(.+)"])) then	
				self.mailUpdateCode = mType_MailCODIncome;
				self:Debug("COD Money Received : !"..self.mailUpdateMoney,1);													
			else -- other money received than from AH must be sent money via mail
				self.mailUpdateCode = mType_MailMoneyReceived;				
				self:Debug("Money received per mail : !"..self.mailUpdateMoney,1);													
			end
		elseif (self.mailData[1].cod > 0) then -- not working since money is picked up b4 the mail is removed
			self.mailUpdateCode = mType_MailCODCost;
			self.mailUpdateMoney = self.mailData[1].cod;
			self:Debug("COD payed : "..self.mailUpdateMoney,1);												
		else
			self:Debug("Unknown mail without money or cod found !",0);
		end
	end		
	self:SaveMailData();
end

function MoneyDetailFu:SaveMailData()
	self.mailData = nil;
	self.mailData = {};
	self:Debug("Mail saved !",3);			
	local index = GetInboxNumItems();	
	if index > 0 then	
		for i=1,index do							
			local _, _, sender, subject, money, CODAmount, _, hasItem, _, _, _, _ = GetInboxHeaderInfo(i);
			if (sender) and ((money > 0) or (CODAmount > 0)) then
				mD = {};
				mD.index = i;
				mD.sender = sender;
				mD.subject = subject;
				mD.money = money;			
				mD.cod = CODAmount;			
				table.insert(self.mailData,mD);
	      self:Debug("Index : "..i.." send : "..sender.." money :"..money.." sender : "..sender,5);				
			end
		end
	end	
end

local GOLD_AMOUNT_inv = GOLD_AMOUNT:gsub("%%d", "%%d+")
local SILVER_AMOUNT_inv = SILVER_AMOUNT:gsub("%%d", "%%d+")
local COPPER_AMOUNT_inv = COPPER_AMOUNT:gsub("%%d", "%%d+")

function MoneyDetailFu:MoneyMsg(coinString)
		local moneys = arg1;
		if (coinString) then
			moneys = coinString;
		end
		local gold = (deformat(moneys:match(GOLD_AMOUNT_inv) or "", GOLD_AMOUNT)) or 0
		local silver = (deformat(moneys:match(SILVER_AMOUNT_inv) or "", SILVER_AMOUNT)) or 0
		local copper = (deformat(moneys:match(COPPER_AMOUNT_inv) or "", COPPER_AMOUNT)) or 0
		
		self.goldLoot = tonumber(gold) * 10000 + tonumber(silver) * 100 + tonumber(copper)
		self:Debug("Money looted "..self.goldLoot,3);
end

function MoneyDetailFu:ChatSystem()
	local msg = arg1;
	if (msg:match(ERR_AUCTION_BID_PLACED)) then
		self:Debug("Bid Placed !",3);	
		self.auctionAction = auction_bidPlaced;
	end
	if (msg:match(ERR_AUCTION_STARTED)) then
		self:Debug("Auction started !",3);	
		self.auctionAction = auction_auctionCreated;
	end	
end

function MoneyDetailFu:AddData(mType, amount)
		local dta = self.db.profile.data[self.realm][self.player];
		if (not dta) then
			return;
		end
		local pMonth = date("%B");
		local pDay = date("%d");
		local pYear = date("%Y");		
		self:Debug("Adding data for date "..pDay,3);		
		if (not dta[pYear]) then
			dta[pYear] = {};
		end		
		if (not dta[pYear][pMonth]) then
			dta[pYear][pMonth] = {};
		end				
		if (not dta[pYear][pMonth][pDay]) then
			dta[pYear][pMonth][pDay] = {};
		end						
		if (not dta[pYear][pMonth][pDay][mType]) or (not dta[pYear][pMonth][pDay][mType].amount) then
			self:Debug("Creating new structure !",4);	
			dta[pYear][pMonth][pDay][mType] = {};
			dta[pYear][pMonth][pDay][mType].amount = amount;
			dta[pYear][pMonth][pDay][mType].count = 1;
			dta[pYear][pMonth][pDay][mType].max = amount;
			dta[pYear][pMonth][pDay][mType].min = amount;
		else
			self:Debug("Adding data !",4);	
			dta[pYear][pMonth][pDay][mType].amount = dta[pYear][pMonth][pDay][mType].amount + amount;
			dta[pYear][pMonth][pDay][mType].count = dta[pYear][pMonth][pDay][mType].count + 1;
			if (amount > dta[pYear][pMonth][pDay][mType].max) then
				dta[pYear][pMonth][pDay][mType].max = amount;
			end
			if (amount < dta[pYear][pMonth][pDay][mType].min) then
				dta[pYear][pMonth][pDay][mType].min = amount;
			end			
		end
end

function MoneyDetailFu:OnClick()
	if (IsAltKeyDown()) then
		self.summary = nil;
	else
		if (not self.ConfigWindow) then
			self:CreateConfigWindow()
		end
		if (self.ConfigWindow:IsVisible()) then
			self.ConfigWindow:Hide();
		else
			self.ConfigWindow:Show();			
		end
	end
end

function MoneyDetailFu:OnTextUpdate()
	if (self.merchant) then
		self:SetText(L["Merchant"]);
	elseif (self.mail) then
		self:SetText(L["Mail"])
	elseif (self.AHOpen) then
		self:SetText(L["AH"])		
	elseif (self.taxi) then
		self:SetText(L["Taxi"])
	elseif (self.trainer) then
		self:SetText(L["Trainer"])		
	elseif (self.talentWipe) then
		self:SetText(L["Unlearn"])		
	else
		self:SetText(abacus:FormatMoneyCondensed(GetMoney(), true))
	end
end

function MoneyDetailFu:FormatData(data)
	self.db.profile.style = "CONDENSED";
	
	local func
	if self.db.profile.style == "CONDENSED" then
		func = abacus.FormatMoneyCondensed
	elseif self.db.profile.style == "SHORT" then
		func = abacus.FormatMoneyShort
	else
		func = abacus.FormatMoneyFull
	end			
	
	local money;
	if (data) then
		money = data.amount;
		if (money == nil) or (money == 0) then
			return "";
		end
		self:Debug("Amount "..money,4);			
	else
		return "";
	end	
	
	moneyStr = func(abacus, money, true);
	moneyStr = moneyStr.."["..data.count.."]";
	
	return moneyStr;
end

local function icaseSort(alpha, bravo)
	if type(alpha) == "number" and type(bravo) == "number" then
		return alpha < bravo
	end
	return tostring(alpha):lower() < tostring(bravo):lower()
end

function MoneyDetailFu:CreateIndexLists()
	self:Debug("Creating Index Lists !",1);	
	self.db.profile.indexPlayers = nil;
	self.db.profile.indexPlayers = {};
	self.db.profile.indexRealms = nil;
	self.db.profile.indexRealms = {};
	for i,_ in pairs(self.db.profile.data) do
		table.insert(self.db.profile.indexRealms,i);		
		self.db.profile.indexPlayers[i] = {};
		for j,_ in pairs(self.db.profile.data[i]) do
			table.insert(self.db.profile.indexPlayers[i],j);		
			table.sort(self.db.profile.indexPlayers[i], icaseSort)
		end					
	end
	self.db.profile.indexDays = nil;
	self.db.profile.indexDays = {};
	self.db.profile.indexMonths = nil;
	self.db.profile.indexMonths = {};	
	for i,_ in pairs(self.db.profile.data) do
		table.insert(self.db.profile.indexMonths,i);		
		self.db.profile.indexDays[i] = {};
		for j,_ in pairs(self.db.profile.data[i]) do
			table.insert(self.db.profile.indexDays[i],j);		
		end					
	end	
	self:ResetIndexes();
end

function MoneyDetailFu:ResetIndexes()
	self:Debug("Resetting Indexes !",1);	
	for i,_ in pairs(self.db.profile.indexRealms) do
		if (self.db.profile.indexRealms[i] == self.realm) then
			self.db.profile.realmIndex = i;
		end
	end
	for j,_ in pairs(self.db.profile.indexPlayers[self.realm]) do
		if (self.db.profile.indexPlayers[self.realm][j] == self.player) then
			self.db.profile.playerIndex = j;
		end
	end						
	
	self.UpdatePlayers();

	self:ResetDateIndexes();
end

function MoneyDetailFu:ResetDateIndexes()
	self:Debug("Resetting Date Indexes !",1);	
	
	self.db.profile.dayIndex =date("%d");
	self.db.profile.monthIndex = date("%B");
	self.db.profile.yearIndex = date("%Y");	
	self.maxMonthDays = MoneyDetailFu_GetMonthMaxDays(self.db.profile.monthIndex,self.db.profile.yearIndex);
	self.UpdateYears();
	self.UpdateMonths();	
end

function MoneyDetailFu:OnTooltipUpdate()
	--if (not IsShiftKeyDown() and not IsControlKeyDown() and not IsAltKeyDown()) then
		
		local cat = tablet:AddCategory(
			'columns', 2,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0
		)
		local tMoney = 0;		
		local line = {};					
		for i,_ in pairs(self.db.profile.characters) do			-- go through realms
			if (self.db.profile.showAllRealmMoney) or (i == self.realm) then
				self:Debug(i,3);
				if (self.db.profile.showAllRealmMoney) then
					line['text'] = i
					cat:AddLine(line);										
				end
				local rMoney = 0;
				for j,_ in pairs(self.db.profile.characters[i]) do			-- go through players				
					if (self.db.profile.showAllRealmMoney) then
						line['text'] = "   "..j
					else
						line['text'] = j
					end
					self:Debug(j,3);
					line['text2'] = abacus.FormatMoneyCondensed(abacus,self.db.profile.characters[i][j].money,true);
					cat:AddLine(line);						
			
					tMoney = tMoney + self.db.profile.characters[i][j].money;
					rMoney = rMoney + self.db.profile.characters[i][j].money;
				end

				if (self.db.profile.showAllRealmMoney) then				
					line['text'] = " "
					line['text2'] = " "
					cat:AddLine(line);						

					line['text'] = "   "..L["Total"];
					line['text2'] = abacus.FormatMoneyCondensed(abacus,rMoney,true);			
					cat:AddLine(line);										
					
					line['text'] = " "
					line['text2'] = " "
					cat:AddLine(line);											
				end
				
			end
		end

		if (self.db.profile.showAllRealmMoney) then				
			line['text'] = L["All Realms"];
		else
			line['text'] = " "
			line['text2'] = " "
			cat:AddLine(line);						
						
			line['text'] = L["Total"];
		end
		line['text2'] = abacus.FormatMoneyCondensed(abacus,tMoney,true);			
		cat:AddLine(line);										
		--return;
	--end
end

function MoneyDetailFu:CalculateAverage(dataSet)
	if (not self.average) or (not dataSet) then
		return dataSet;
	end
	self.avDataSet = nil;
	self.avDataSet = dataSet;
	for j,_ in pairs(self.avDataSet) do -- go through all money data types
		if ((self.avDataSet[j]) and (self.avDataSet[j].amount) and (self.avDataSet[j].count)) then 					
			self.avDataSet[j].amount = self.avDataSet[j].amount / self.avDataSet[j].count;			
		end
	end
	return self.avDataSet;
end

function MoneyDetailFu:CalculateSummaries()
	local pRealm = self.db.profile.indexRealms[self.db.profile.realmIndex];
	local pPlayer = self.db.profile.indexPlayers[pRealm][self.db.profile.playerIndex];	
	if (not self.summary) then
		self.summary = {};
	end	
	if (not self.summary[pRealm]) then
		self.summary[pRealm] = {};
	end	
	for i,_ in pairs(self.db.profile.data[pRealm]) do	
		if (not self.summary[pRealm][i]) then
			self.summary[pRealm][i] = {};
		end		
		if (not self.summary[pRealm][i][self.db.profile.yearIndex]) then
			self.summary[pRealm][i][self.db.profile.yearIndex] = {};
		end				
		if (not self.summary[pRealm][i][self.db.profile.yearIndex][self.db.profile.monthIndex]) then
			self.summary[pRealm][i][self.db.profile.yearIndex][self.db.profile.monthIndex] = {};
		end				
	end
	
	self.summary.summDays = nil;
	self.summary.summPlayers = nil;
	self.summary.summMonths = nil;	
	
	if (tonumber(self.db.profile.playerIndex) == const_allPlayers) then -- all players requested
		self:Debug("All Players !",5);	
		return self:playerSummary(pRealm,pPlayer,self.db.profile.yearIndex,self.db.profile.monthIndex,self.db.profile.dayIndex);
	elseif (self.db.profile.monthIndex == L["All Months"]) then					
		self:Debug("Ordering Year Summary !",5);	
		return self:yearSummary(pRealm,pPlayer,self.db.profile.yearIndex);
	elseif (self.db.profile.dayIndex == L["All Days"]) then		
		self:Debug("Ordering Month Summary !"..self.db.profile.monthIndex,5);			
		return self:monthSummary(pRealm,pPlayer,self.db.profile.yearIndex,self.db.profile.monthIndex);
	end
	return nil;
end

function MoneyDetailFu:playerSummary(pRealm,pPlayer,pYear,pMonth,pDay)
	if (not self.summary[pRealm][pYear]) then
		self.summary[pRealm][pYear] = {};
	end	
	if (not self.summary[pRealm][pYear][pMonth]) then
		self.summary[pRealm][pYear][pMonth] = {};
	end		
	if (self.db.profile.monthIndex == L["All Months"]) then					
		for i,_ in pairs(self.db.profile.data[pRealm]) do
		end
	elseif (self.db.profile.dayIndex == L["All Days"]) then					
		self:Debug("Ordering Month Summary !",5);	
		self.summary[pRealm][pYear][pMonth].s = nil;	
		self.summary[pRealm][pYear][pMonth].s = {};	
		self.summary[pRealm][pYear][pMonth].s.summPlayers = nil;
		self.summary[pRealm][pYear][pMonth].s.summPlayers = {};							
		self.summary[pRealm][pYear][pMonth].s.summDays = nil;
		self.summary[pRealm][pYear][pMonth].s.summDays = {};									
		for i,_ in pairs(self.db.profile.data[pRealm]) do
			if (self.db.profile.data[pRealm][i][pYear]) and (self.db.profile.data[pRealm][i][pYear][pMonth]) then
				self:Debug("Player data found ! "..i,5);	
				self:monthSummary(pRealm,i,pYear,pMonth);				
				table.insert(self.summary[pRealm][pYear][pMonth].s.summPlayers,i);
				self:addSummaryData(self.summary[pRealm][pYear][pMonth],self.summary[pRealm][i][pYear][pMonth].s);							
				self:mergeSummValues(self.summary[pRealm][pYear][pMonth].s.summDays,self.summary[pRealm][i][pYear][pMonth].s.summValues);
			end			
		end		
		self.summary.summDays = self.summary[pRealm][pYear][pMonth].s.summDays;
		self.summary.summPlayers = self.summary[pRealm][pYear][pMonth].s.summPlayers;		
		return self.summary[pRealm][pYear][pMonth].s;
	else
		self:Debug("Ordering Day Summary !",5);	
		self.summary[pRealm][pYear][pMonth][pDay] = nil;
		self.summary[pRealm][pYear][pMonth][pDay] = {};
		self.summary[pRealm][pYear][pMonth][pDay].s = nil;	
		self.summary[pRealm][pYear][pMonth][pDay].s = {};	
		self.summary[pRealm][pYear][pMonth][pDay].s.summPlayers = nil;
		self.summary[pRealm][pYear][pMonth][pDay].s.summPlayers = {};					
		for i,_ in pairs(self.db.profile.data[pRealm]) do			
			if (self.db.profile.data[pRealm][i][pYear]) and (self.db.profile.data[pRealm][i][pYear][pMonth]) and (self.db.profile.data[pRealm][i][pYear][pMonth][pDay]) then
				self:Debug("Player data found ! "..i,5);	
				table.insert(self.summary[pRealm][pYear][pMonth][pDay].s.summPlayers,i);
				self:addSummaryData(self.summary[pRealm][pYear][pMonth][pDay],self.db.profile.data[pRealm][i][pYear][pMonth][pDay]);							
			end
		end		
		self.summary.summDays = nil;
		self.summary.summPlayers = self.summary[pRealm][pYear][pMonth][pDay].s.summPlayers;				
		return self.summary[pRealm][pYear][pMonth][pDay].s;
	end	
end


function MoneyDetailFu:yearSummary(pRealm,pPlayer,pYear)
	if (self.db.profile.data[pRealm][pPlayer][pYear]) then
		for i,_ in pairs(self.db.profile.data[pRealm][pPlayer][pYear]) do
			self:monthSummary(pRealm,pPlayer,pYear,i);	
		end	
		self:createStatSummary(self.summary[pRealm][pPlayer],self.summary[pRealm][pPlayer],pYear,1)	
		if (self.summary[pRealm][pPlayer][pYear].s) then		
			self.summary.summDays = nil;  -- this is more complicated now
			self.summary.summPlayers = nil;
			self.summary.summMonths = self.summary[pRealm][pPlayer][pYear].s.summValues;		
		end
		
		return self.summary[pRealm][pPlayer][pYear].s;
	end
end

function MoneyDetailFu:monthSummary(pRealm,pPlayer,pYear,pMonth)
	self.summary[pRealm][pPlayer][pYear][pMonth] = nil;
	self.summary[pRealm][pPlayer][pYear][pMonth] = {};
	
	self:createStatSummary(self.db.profile.data[pRealm][pPlayer][pYear],self.summary[pRealm][pPlayer][pYear],pMonth)
	if (self.summary[pRealm][pPlayer][pYear][pMonth].s) then	
		self.summary.summDays = self.summary[pRealm][pPlayer][pYear][pMonth].s.summValues;
		self.summary.summPlayers = nil;
		self.summary.summMonths = nil;			
	end
	
	return self.summary[pRealm][pPlayer][pYear][pMonth].s;	
end

function MoneyDetailFu:mergeSummValues(dest,source) 
	if (not source) or (not dest) then
		return;
	end
	for i,_ in pairs(source) do			
		local found = false;
		for j,_ in pairs(dest) do			
			if (source[i] == dest[j]) then
				found = true;
				break;
			end
		end			
		if (found == false) then
			table.insert(dest,source[i]);
		end		
	end
end

function MoneyDetailFu:addSummaryData(dest,sData) 
	if (sData) then
		for j,_ in pairs(sData) do -- go through all money data types
			if ((sData[j]) and (sData[j].amount)) then 					
				if (not dest.s[j]) then
					dest.s[j] = {};
					dest.s[j].amount = 0;
					dest.s[j].count = 0;
					dest.s[j].max = 0;
					dest.s[j].min = 999999;							
				end
				dest.s[j].amount = dest.s[j].amount + sData[j].amount;
				dest.s[j].count = dest.s[j].count + sData[j].count;
				if (sData[j].max > dest.s[j].max) then
					dest.s[j].max = sData[j].max;
				end
				if (sData[j].min < dest.s[j].min) then
					dest.s[j].min = sData[j].min;
				end		
			end
		end
	end
end

function MoneyDetailFu:createStatSummary(source,dest,index,sourceIsSummary)
	if (not source) or (not source[index]) then
		return nil;
	end
	self:Debug("Creating summary for !"..index,2);	
	if (not dest[index]) then
		dest[index] = {};	
	end
	dest[index].s = nil;	
	dest[index].s = {};	
	dest[index].s.summValues = nil;
	dest[index].s.summValues = {};
--	dest[index].s = nil;
--	
--	dest[index].s.amount = 0;
---	dest[index].s.count = 0;
--	dest[index].s.max = 0;
--	dest[index].s.min = 999999;	
	for i,_ in pairs(source[index]) do
		self:Debug("Checking !"..i,5);	
		if (not (i == "s")) and (source[index][i]) then -- go through all subIndexes
			self:Debug("OK : !"..i,4);	
			--dest[index].s.summAmount = dest[index].s.summAmount + 1; -- 1 more data source added
			table.insert(dest[index].s.summValues,i);
			local sData = source[index][i];
			if (sourceIsSummary) then -- if the source is also a summary we have to access the summary part
				sData = source[index][i].s;
				self:Debug("SourceIsSummary : !",5);	
			end
			self:addSummaryData(dest[index],sData);			
		end
	end	
end

function MoneyDetailFu:getDataSetAmount(dataSet)
	if (dataSet) and (dataSet.amount) then
		return dataSet.amount;
	else
		return 0;
	end
end

--[[function MoneyDetailFu:yearSummary(pRealm,pPlayer,pYear)
	if (self.db.profile.data[pRealm][pPlayer][pYear]) then
		for i,_ in pairs(self.db.profile.data[pRealm][pPlayer][pYear]) do
			self.monthSummary(pRealm,pPlayer,pYear,i);	
		end		
		self.createStatSummary(self.summary[pRealm][pPlayer],self.summary[pRealm][pPlayer],pYear,1);	
		return self.summary[pRealm][pPlayer][pYear].s;
	end
end

]]--