local L = AceLibrary("AceLocale-2.2"):new("FuBar_MoneyDetailFu")
local abacus = LibStub("LibAbacus-3.0")

local mType_Loot = 1;
local mType_Repair = 2;
local mType_Buy = 3;
local mType_Sell = 4;
local mType_Quest = 5;
local mType_MailSent = 6;
local mType_MailCost = 7;
local mType_MailCODCost = 13;
local mType_MailCODIncome = 15;
local mType_MailMoneyReceived = 14;
local mType_Fly = 8;
local mType_AuctionCost = 9;
local mType_AuctionBid = 10;
local mType_AuctionOutbid = 11;
local mType_AuctionSold = 12;
local mType_Talents = 16;

local const_allPlayers = 20; -- constant passed as index for all players

------------------------------------
--Frame Implementation
------------------------------------

function MoneyDetailFu:CreateConfigWindow()
	local theFrame=CreateFrame("Frame", Name,UIParent)

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent,"TOP",0,-300)
	theFrame:SetHeight(500)
	theFrame:SetWidth(580)

	theFrame:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 5, bottom = 3 }
	})
	theFrame:SetBackdropBorderColor(1.0,0.0,0.0)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255)

	--Recount.Colors:RegisterBorder("Window","Title",theFrame)
	--Recount.Colors:RegisterBackground("Window","Background",theFrame)
	
	theFrame:EnableMouse(true)
	theFrame:SetMovable(true)
	theFrame.isLocked = nil;

	theFrame:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  --Recount:SetWindowTop(this)
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
	theFrame:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;

						  --if this.SavePosition then
							--this:SavePosition()
						  --end
						 end
						end)
	theFrame:SetScript("OnShow", function()
							MoneyDetailFu:UpdateTexts();
						end)
	theFrame:SetScript("OnHide", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)
	theFrame.Title:SetTextColor(1.0,1.0,1.0,1.0)
	theFrame.Title:SetText("")
	--Recount:AddFontString(theFrame.Title)

	theFrame.CloseButton=CreateFrame("Button",nil,theFrame)
	theFrame.CloseButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up.blp")
	theFrame.CloseButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down.blp")
	theFrame.CloseButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	theFrame.CloseButton:SetWidth(20)
	theFrame.CloseButton:SetHeight(20)
	theFrame.CloseButton:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",0,0)
	theFrame.CloseButton:SetScript("OnClick",function() this:GetParent():Hide() end)
	
	theFrame.TodayButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.TodayButton:SetWidth(100)
	theFrame.TodayButton:SetHeight(18)
	theFrame.TodayButton:SetPoint("TOPLEFT",theFrame,"TOPLEFT",20,-4)
	theFrame.TodayButton:SetScript("OnClick",function() MoneyDetailFu:ResetDateIndexes() end)
	theFrame.TodayButton:SetText(L["Today"])	
	
	theFrame.AverageTotalButton=CreateFrame("Button",nil,theFrame,"OptionsButtonTemplate")
	theFrame.AverageTotalButton:SetWidth(100)
	theFrame.AverageTotalButton:SetHeight(18)
	theFrame.AverageTotalButton:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-150,-4)
	theFrame.AverageTotalButton:SetScript("OnClick",function() MoneyDetailFu:AverageTotalToggle() end)
	theFrame.AverageTotalButton:SetText(L["Average"])		
	
	self:CreatePlayerSelection(theFrame);
	self:CreateMonthSelection(theFrame);
	self:CreateDaySelection(theFrame);
	self:CreateYearSelection(theFrame);
	self:CreateDisplayTexts(theFrame);
	self:UpdateTexts();
	
	theFrame:Hide();
	
	self.ConfigWindow	= theFrame;	
end

function MoneyDetailFu:CreatePlayerLine(parent,player)
	local frame=CreateFrame("Frame",nil,parent)
	frame:SetHeight(13)
	frame:SetWidth(80)
	frame.Text=frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frame.Text:SetPoint("CENTER",frame,"CENTER")
	frame.Texture=frame:CreateTexture(nil,"BACKGROUND")
	frame.Texture:SetAllPoints(frame)
	frame.Texture:SetTexture(1,1,1,0.5)
	if (tonumber(player) < const_allPlayers) then
		frame.Text:SetText(self.db.profile.indexPlayers[self.realm][player])
	else
		frame.Text:SetText(L["All Players"])
	end
	frame.Text:SetFont("GameFontNormal",12)
	frame.SetTo=player	
	frame:EnableMouse()
	frame:SetScript("OnMouseDown",function() MoneyDetailFu.db.profile.playerIndex = this.SetTo 
						MoneyDetailFu:UpdatePlayers() end)
	frame:Show()
	return frame
end

function MoneyDetailFu:UpdatePlayers()
	if (not self) then
		return;
	end
	for _, v in pairs(self.PlayerOptions.Rows) do
		if v.SetTo==MoneyDetailFu.db.profile.playerIndex then
			v.Texture:SetVertexColor(0.2,0.9,0.2)
		else
			v.Texture:SetVertexColor(0.9,0.2,0.2)
		end
	end
	self:UpdateTexts();
end

function MoneyDetailFu:CreatePlayerSelection(parent)
	self.PlayerOptions=CreateFrame("Frame",nil,parent)

	local theFrame=self.PlayerOptions
	--local Fonts=SM:List("font")

	theFrame:SetHeight(200)
	theFrame:SetWidth(80)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",10,-20)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText(L["Player"])
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)	

	theFrame.Rows={}
	local i = 1;
	for j,_ in pairs(self.db.profile.indexPlayers[self.realm]) do
		theFrame.Rows[i]=self:CreatePlayerLine(theFrame,j)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)
		--theFrame.Rows[i]:SetFont("GameFontNormal",12)		
		i = i +1;
	end					
	theFrame.Rows[i]=self:CreatePlayerLine(theFrame,const_allPlayers)
	theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)	
	
	self:UpdatePlayers()
end

function MoneyDetailFu:CreateMonthLine(parent,month)
	local frame=CreateFrame("Frame",nil,parent)
	frame:SetHeight(13)
	frame:SetWidth(80)
	frame.Text=frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frame.Text:SetPoint("CENTER",frame,"CENTER")
	frame.Texture=frame:CreateTexture(nil,"BACKGROUND")
	frame.Texture:SetAllPoints(frame)
	frame.Texture:SetTexture(1,1,1,0.5)
	frame.Text:SetText(month)
	frame.Text:SetFont("GameFontNormal",12)
	frame.SetTo=month	
	frame:EnableMouse()
	frame:SetScript("OnMouseDown",function() MoneyDetailFu.db.profile.monthIndex = this.SetTo 
						MoneyDetailFu:UpdateMonths() end)
	frame:Show()
	return frame
end

function MoneyDetailFu:UpdateMonths()
	if (not MoneyDetailFu.MonthOptions) then
		return;
	end
	for _, v in pairs(MoneyDetailFu.MonthOptions.Rows) do
		if v.SetTo==MoneyDetailFu.db.profile.monthIndex then
			v.Texture:SetVertexColor(0.2,0.9,0.2)
		else
			v.Texture:SetVertexColor(0.9,0.2,0.2)
		end
	end
	MoneyDetailFu.maxMonthDays = MoneyDetailFu_GetMonthMaxDays(MoneyDetailFu.db.profile.monthIndex,MoneyDetailFu.db.profile.yearIndex);		
	MoneyDetailFu:UpdateDays(); -- update the amount of days shown
	--self:UpdateTexts(); will be called in updatedays
end

function MoneyDetailFu:CreateMonthSelection(parent)
	self.MonthOptions=CreateFrame("Frame",nil,parent)

	local theFrame=self.MonthOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(80)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",10,-170)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText(L["Month"])
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)	

	theFrame.Rows={}
	local i = 1;
	for j =1,12 do
		theFrame.Rows[i]=self:CreateMonthLine(theFrame,MoneyDetailFu_GetMonthName(i))
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)
		--theFrame.Rows[i]:SetFont("GameFontNormal",12)		
		i = i +1;
	end					
	theFrame.Rows[i]=self:CreateMonthLine(theFrame,L["All Months"])
	theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)	
	theFrame.Rows[i]:Hide(); -- not implemented yet
	
	self:UpdateMonths()
end

function MoneyDetailFu:CreateDayLine(parent,day)
	local frame=CreateFrame("Frame",nil,parent)
	frame:SetHeight(13)
	frame:SetWidth(50)
	frame.Text=frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frame.Text:SetPoint("CENTER",frame,"CENTER")
	frame.Texture=frame:CreateTexture(nil,"BACKGROUND")
	frame.Texture:SetAllPoints(frame)
	frame.Texture:SetTexture(1,1,1,0.5)
	frame.Text:SetText(day)
	frame.Text:SetFont("GameFontNormal",12)
	frame.SetTo=tostring(day)
	frame:EnableMouse()
	frame:SetScript("OnMouseDown",function() MoneyDetailFu.db.profile.dayIndex = this.SetTo 
						MoneyDetailFu:UpdateDays() end)
	frame:Show()
	return frame
end

function MoneyDetailFu:UpdateDays()
	if (not MoneyDetailFu.DayOptions) then
		MoneyDetailFu:Debug("no dayoptions !",5)
		return;
	end
	MoneyDetailFu:Debug("Updating Day Display !",3)
	for i, v in pairs(MoneyDetailFu.DayOptions.Rows) do
		if v.SetTo==MoneyDetailFu.db.profile.dayIndex then
			v.Texture:SetVertexColor(0.2,0.9,0.2)
		else
			v.Texture:SetVertexColor(0.9,0.2,0.2)
		end
		if (tonumber(i) <= MoneyDetailFu.maxMonthDays) or (i ==32) then		
			v:Show();
		else
			v:Hide();
		end
	end
	MoneyDetailFu:UpdateTexts();	
end

function MoneyDetailFu:CreateDaySelection(parent)
	self.DayOptions=CreateFrame("Frame",nil,parent)

	local theFrame=self.DayOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(50)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",100,-20)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText(L["Day"])
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)	

	theFrame.Rows={}
	for i =1,31 do
		theFrame.Rows[i]=self:CreateDayLine(theFrame,i)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)
		--theFrame.Rows[i]:SetFont("GameFontNormal",12)		
	end					
	i = 32;
	theFrame.Rows[i]=self:CreateDayLine(theFrame,L["All Days"])
	theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)	
	
	self:UpdateDays()
end

function MoneyDetailFu:CreateYearLine(parent,year)
	local frame=CreateFrame("Frame",nil,parent)
	frame:SetHeight(13)
	frame:SetWidth(80)
	frame.Text=frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frame.Text:SetPoint("CENTER",frame,"CENTER")
	frame.Texture=frame:CreateTexture(nil,"BACKGROUND")
	frame.Texture:SetAllPoints(frame)
	frame.Texture:SetTexture(1,1,1,0.5)
	frame.Text:SetText(year)
	frame.Text:SetFont("GameFontNormal",12)
	frame.SetTo=tostring(year)
	frame:EnableMouse()
	frame:SetScript("OnMouseDown",function() MoneyDetailFu.db.profile.yearIndex = this.SetTo 
						MoneyDetailFu:UpdateYears() end)
	frame:Show()
	return frame
end

function MoneyDetailFu:UpdateYears()
	if (not MoneyDetailFu.YearOptions) then
		return;
	end
	for _, v in pairs(MoneyDetailFu.YearOptions.Rows) do
		if v.SetTo==MoneyDetailFu.db.profile.yearIndex then
			v.Texture:SetVertexColor(0.2,0.9,0.2)
		else
			v.Texture:SetVertexColor(0.9,0.2,0.2)
		end
	end
	MoneyDetailFu.maxMonthDays = MoneyDetailFu_GetMonthMaxDays(MoneyDetailFu.db.profile.monthIndex,MoneyDetailFu.db.profile.yearIndex);		
	MoneyDetailFu:UpdateDays(); -- update the amount of days shown	
	--self:UpdateTexts(); will be called in updatedays
end

function MoneyDetailFu:CreateYearSelection(parent)
	self.YearOptions=CreateFrame("Frame",nil,parent)

	local theFrame=self.YearOptions

	theFrame:SetHeight(200)
	theFrame:SetWidth(80)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",10,-370)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetText(L["Year"])
	theFrame.Title:SetPoint("TOP",theFrame,"TOP",0,-2)	

	theFrame.Rows={}
	local thisYear = self.db.profile.yearIndex; -- index is a string
	local i=1;
	for j =thisYear-5,thisYear do
		theFrame.Rows[i]=self:CreateYearLine(theFrame,j)
		theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)
		--theFrame.Rows[i]:SetFont("GameFontNormal",12)		
		i = i+1;
	end					
	theFrame.Rows[i]=self:CreateYearLine(theFrame,L["All Years"])
	theFrame.Rows[i]:SetPoint("TOP",theFrame,"TOP",0,-i*14-2)	
	
	theFrame.Rows[i]:Hide(); -- not implemented yet
	
	self:UpdateYears()
end

function MoneyDetailFu:CreateSingleDisplayText(parent,index,x,y,text,r,g,b,a)
	parent.displayStrings[index]=parent:CreateFontString(nil,"OVERLAY","GameFontNormal")
	parent.displayStrings[index]:SetText(text)
	parent.displayStrings[index]:SetPoint("TOP",parent,"TOP",x,y)	
	parent.displayStrings[index]:SetTextColor(r,g,b,a)
end

function MoneyDetailFu:CreateDisplayTexts(parent)
	self.DisplayTexts=CreateFrame("Frame",nil,parent)

	local theFrame=self.DisplayTexts
	
	local column0 = 0;
	local column1 = 100;
	local column2 = 200;
	local column3 = 300;
	local rowSpace = 20;
	local emptyString = "";

	theFrame:SetHeight(500)
	theFrame:SetWidth(400)
	theFrame:SetPoint("TOPLEFT",parent,"TOPLEFT",20,-20)
	
	-- id < 100 amounts filled by ids
	-- id 100 - 199 fixed strings
	-- id 200 + placeholders for some to come

	local y = 0;
	theFrame.displayStrings = {};
	self:CreateSingleDisplayText(theFrame,100,column0,y,L["Incoming"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,101,column1,y,L["Amount"],1,1,1,1);
	self:CreateSingleDisplayText(theFrame,102,column2,y,L["Amount"],1,1,1,1);
	self:CreateSingleDisplayText(theFrame,103,column3,y,L["Outgoing"],1,0,0,1);	
	
	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,104,column0,y,L["Loot"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_Loot,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_Repair,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,105,column3,y,L["Repairs"],1,0,0,1);		
	
	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,106,column0,y,L["Quests"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_Quest,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_Fly,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,107,column3,y,L["Flights"],1,0,0,1);		

	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,108,column0,y,L["Selling"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_Sell,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_Buy,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,109,column3,y,L["Buying"],1,0,0,1);		

	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,110,column0,y,L["Outbid Aucts"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_AuctionOutbid,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_AuctionBid,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,111,column3,y,L["Auction Bids"],1,0,0,1);		

	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,112,column0,y,L["Sold Auctions"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_AuctionSold,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_AuctionCost,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,113,column3,y,L["Auction Costs"],1,0,0,1);		
	
	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,200,column0,y,emptyString,0,1,0,1);
	self:CreateSingleDisplayText(theFrame,201,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_MailCost,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,115,column3,y,L["Mail Costs"],1,0,0,1);		

	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,116,column0,y,L["Inc MMails"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_MailMoneyReceived,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_MailSent,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,117,column3,y,L["Out MMails"],1,0,0,1);		

	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,118,column0,y,L["COD Payment"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,mType_MailCODIncome,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_MailCODCost,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,119,column3,y,L["COD payed"],1,0,0,1);		
	
	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,202,column0,y,emptyString,0,1,0,1);
	self:CreateSingleDisplayText(theFrame,203,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,mType_Talents,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,120,column3,y,L["Train/Unlearn"],1,0,0,1);		
	
	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,121,column0,y,L["Total"],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,204,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,205,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,122,column3,y,L["Total"],1,0,0,1);		
		
	y = y - rowSpace;
	self:CreateSingleDisplayText(theFrame,123,column0,y,L["Summary for : "],0,1,0,1);
	self:CreateSingleDisplayText(theFrame,206,column1,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,207,column2,y,emptyString,1,1,1,1);
	self:CreateSingleDisplayText(theFrame,208,column3,y,emptyString,1,1,1,1);			
	self.DisplayTexts.displayStrings[123]:Hide();
end

function MoneyDetailFu:UpdateTexts()

	if (not self.DisplayTexts) then
		return;
	end
	
	local pMonth = date("%B");
	local pDay = date("%d");
	local pYear = date("%Y");		
	local pPlayer = self.db.profile.indexPlayers[self.realm][self.db.profile.playerIndex];	
	local dataSet = self:CalculateSummaries();
	if (not dataSet) then
		dataSet = self.db.profile.data[self.realm][pPlayer];
		if (dataSet) then
			dataSet = dataSet[self.db.profile.yearIndex];
		end
		if (dataSet) then
			dataSet = dataSet[self.db.profile.monthIndex];
		end
		if (dataSet) then
			dataSet = dataSet[self.db.profile.dayIndex];
		end		
	end
	dataSet = self:CalculateAverage(dataSet);
	
	if (not dataSet) then
		for i=1,16 do
			self.DisplayTexts.displayStrings[i]:SetText("");
		end		
		self.DisplayTexts.displayStrings[204]:SetText(abacus.FormatMoneyFull(abacus,0,true));
		self.DisplayTexts.displayStrings[205]:SetText(abacus.FormatMoneyFull(abacus,0,true));
		
		return;
	end

	for i=1,16 do
		if (dataSet[i]) then
			self.DisplayTexts.displayStrings[i]:SetText(MoneyDetailFu:FormatData(dataSet[i]));
		else
			self.DisplayTexts.displayStrings[i]:SetText("");
		end
	end
	
	local totalInc = self:getDataSetAmount(dataSet[mType_Loot]) + self:getDataSetAmount(dataSet[mType_Quest]) + self:getDataSetAmount(dataSet[mType_Sell]) + self:getDataSetAmount(dataSet[mType_AuctionOutbid]) + self:getDataSetAmount(dataSet[mType_AuctionSold]) + self:getDataSetAmount(dataSet[mType_MailMoneyReceived]) + self:getDataSetAmount(dataSet[mType_MailCODIncome]);
	local totalOut = self:getDataSetAmount(dataSet[mType_Repair]) + self:getDataSetAmount(dataSet[mType_Fly]) + self:getDataSetAmount(dataSet[mType_Buy]) + self:getDataSetAmount(dataSet[mType_AuctionBid]) + self:getDataSetAmount(dataSet[mType_AuctionCost]) + self:getDataSetAmount(dataSet[mType_MailCost]) + self:getDataSetAmount(dataSet[mType_MailSent]) + self:getDataSetAmount(dataSet[mType_MailCODCost]) + self:getDataSetAmount(dataSet[mType_Talents]);
	self.DisplayTexts.displayStrings[204]:SetText(abacus.FormatMoneyFull(abacus,totalInc,true));
	self.DisplayTexts.displayStrings[205]:SetText(abacus.FormatMoneyFull(abacus,totalOut,true));
	self:Debug("Total Inc : "..totalInc,2);	
	self:Debug("Total out : "..totalOut,2);		
	
	if (tonumber(self.db.profile.playerIndex) == const_allPlayers) or		-- all players requested
		(self.db.profile.monthIndex == L["All Months"])  or -- all months requested
		(self.db.profile.dayIndex == L["All Days"]) -- all days requested
		then
		self.DisplayTexts.displayStrings[123]:Show();
	
		if (self.summary.summDays) then
			self.DisplayTexts.displayStrings[206]:SetText(""..table.getn(self.summary.summDays)..L[" days"])
		else
			self.DisplayTexts.displayStrings[206]:SetText(L["1 day"]);
		end
		if (self.summary.summMonths) then
			self.DisplayTexts.displayStrings[207]:SetText(""..table.getn(self.summary.summMonths)..L[" months"]);
		else
			self.DisplayTexts.displayStrings[207]:SetText(L["1 month"]);
		end
		if (self.summary.summPlayers) then
			self.DisplayTexts.displayStrings[208]:SetText(""..table.getn(self.summary.summPlayers)..L[" players"]);
		else
			self.DisplayTexts.displayStrings[208]:SetText(L["1 player"]);
		end		
	else
		self.DisplayTexts.displayStrings[123]:Hide();
		self.DisplayTexts.displayStrings[206]:SetText("");
		self.DisplayTexts.displayStrings[207]:SetText("");
		self.DisplayTexts.displayStrings[208]:SetText("");		
	end
end

function MoneyDetailFu:AverageTotalToggle()
	if (MoneyDetailFu.average) then
		MoneyDetailFu.average = nil;
		MoneyDetailFu.ConfigWindow.AverageTotalButton:SetText(L["Average"])		
	else
		MoneyDetailFu.average = 1;
		MoneyDetailFu.ConfigWindow.AverageTotalButton:SetText(L["Total"])				
	end	
	self:UpdateTexts();
end